﻿namespace Invadurz

[<AutoOpen>]
module BitField =
    open System.Windows.Media
    open System.Windows.Media.Imaging
    let toBitmap (width,xs:int []) =
        let bitmap = WriteableBitmap(width,xs.Length)
        let pixels = bitmap.Pixels
        xs |> Array.iteri (fun y xs ->
            for x = 0 to width-1 do
                let bit = 1 <<< (width - 1 - x) 
                pixels.[x+y*width] <-
                    xs &&& bit = bit
                    |> (function true -> 0xffffffff | false -> 0x00000000)
        )
        bitmap

open System.Windows
open System.Windows.Controls

type Sprite(elements:UIElement seq,x,y,animation:seq<unit>) as canvas =
    inherit Canvas ()
    let e = animation.GetEnumerator()
    let setPosition(x,y) = 
        Canvas.SetLeft(canvas,x)
        Canvas.SetTop(canvas,y)
    do  setPosition(x,y)
    do  elements |> Seq.iter canvas.Children.Add
    new (element:UIElement) = Sprite([element],0.0,0.0)
    new (element:UIElement,x,y) = Sprite([element],x,y,[])
    new (elements:UIElement seq,x,y) = Sprite(elements,x,y,[])
    member this.Animate() = e.MoveNext() |> ignore
    member this.MoveTo (x,y) = setPosition(x,y)
    member this.X = Canvas.GetLeft(canvas)
    member this.Y = Canvas.GetTop(canvas)
    member this.HitTest(x,y) =
        let x',y' = this.X, this.Y
        elements |> Seq.exists (fun child ->
            let w = child.GetValue(FrameworkElement.WidthProperty) :?> float
            let h = child.GetValue(FrameworkElement.HeightProperty) :?> float
            x >= x' && x < x' + w &&
            y >= y' && y < y' + h
        )